@@ -1,6 +1,6 @@
 /*
  * POSIX library for Lua 5.1, 5.2, 5.3 & 5.4.
- * Copyright (C) 2013-2021 Gary V. Vaughan
+ * Copyright (C) 2013-2023 Gary V. Vaughan
  * Copyright (C) 2010-2013 Reuben Thomas <rrt@sc3d.org>
  * Copyright (C) 2008-2010 Natanael Copa <natanael.copa@gmail.com>
  * Clean up and bug fixes by Leo Razoumov <slonik.az@gmail.com> 2006-10-11
@@ -103,7 +103,11 @@ pushsockaddrinfo(lua_State *L, int family, struct sockaddr *sa)
 		case AF_UNIX:
 		{
 			struct sockaddr_un *sau = (struct sockaddr_un *)sa;
-			pushlstringfield("path", sau->sun_path, sizeof (sau->sun_path));
+			size_t path_len = sizeof sau->sun_path;
+			char *end = memchr(sau->sun_path, 0, path_len);
+			if (end)
+				path_len = end - sau->sun_path;
+			pushlstringfield("path", sau->sun_path, path_len);
 			break;
 		}
 #if HAVE_LINUX_NETLINK_H
@@ -241,8 +245,8 @@ sockaddr_from_lua(lua_State *L, int index, struct sockaddr_storage *sa, socklen_
 			int port		= checkintfield(L, index, "port");
 			const char *addr	= checkstringfield(L, index, "addr");
 			/* For compatibility, the next two fields are optional */
-			uint32_t flowinfo	= optintfield(L, index, "flowinfo", 0);
-			uint32_t scope_id	= optintfield(L, index, "scope_id", 0);
+			uint32_t flowinfo	= (uint32_t)optintegerfield(L, index, "flowinfo", 0);
+			uint32_t scope_id	= (uint32_t)optintegerfield(L, index, "scope_id", 0);
 
 			checkfieldnames (L, index, Safinet6_fields);
 
@@ -532,7 +536,7 @@ static int
 Precv(lua_State *L)
 {
 	int fd = checkint(L, 1);
-	int count = checkint(L, 2), ret;
+	size_t count = (size_t)checkinteger(L, 2), ret;
 	void *ud, *buf;
 	lua_Alloc lalloc;
 
@@ -577,7 +581,7 @@ Precvfrom(lua_State *L)
 	struct sockaddr_storage sa;
 	int r;
 	int fd = checkint(L, 1);
-	int count = checkint(L, 2);
+	size_t count = (size_t)checkinteger(L, 2);
 	lua_Alloc lalloc;
 
 	checknargs(L, 2);
@@ -720,8 +724,8 @@ Psetsockopt(lua_State *L)
 				case SO_LINGER:
 					checknargs(L, 5);
 
-					linger.l_onoff = checkint(L, 4);
-					linger.l_linger = checkint(L, 5);
+					linger.l_onoff = checkinteger(L, 4);
+					linger.l_linger = checkinteger(L, 5);
 					val = &linger;
 					len = sizeof(linger);
 					break;
@@ -729,8 +733,8 @@ Psetsockopt(lua_State *L)
 				case SO_SNDTIMEO:
 					checknargs(L, 5);
 
-					tv.tv_sec = checkint(L, 4);
-					tv.tv_usec = checkint(L, 5);
+					tv.tv_sec = checkinteger(L, 4);
+					tv.tv_usec = checkinteger(L, 5);
 					val = &tv;
 					len = sizeof(tv);
 					break;
@@ -784,7 +788,7 @@ Psetsockopt(lua_State *L)
 
 	if (val == NULL)
 	{
-		vint = checkint(L, 4);
+		vint = checkinteger(L, 4);
 		val = &vint;
 		len = sizeof(vint);
 	}
@@ -913,7 +917,7 @@ static int Pgetsockname(lua_State *L)
 {
 	int fd = checkint(L, 1);
 	struct sockaddr_storage sa;
-	socklen_t salen;
+	socklen_t salen = sizeof sa;
 	checknargs (L, 1);
 	if (getsockname(fd, (struct sockaddr *)&sa, &salen) != 0)
 		return pusherror(L, "getsockname");
@@ -936,7 +940,7 @@ static int Pgetpeername(lua_State *L)
 {
 	int fd = checkint(L, 1);
 	struct sockaddr_storage sa;
-	socklen_t salen;
+	socklen_t salen = sizeof sa;
 	checknargs (L, 1);
 	if (getpeername(fd, (struct sockaddr *)&sa, &salen) != 0)
 		return pusherror(L, "getpeername");
