@@ -1,6 +1,6 @@
 /*
  * POSIX library for Lua 5.1, 5.2, 5.3 & 5.4.
- * Copyright (C) 2013-2021 Gary V. Vaughan
+ * Copyright (C) 2013-2023 Gary V. Vaughan
  * Copyright (C) 2010-2013 Reuben Thomas <rrt@sc3d.org>
  * Copyright (C) 2008-2010 Natanael Copa <natanael.copa@gmail.com>
  * Clean up and bug fixes by Leo Razoumov <slonik.az@gmail.com> 2006-10-11
@@ -40,7 +40,7 @@ static int
 Pwait(lua_State *L)
 {
 	int status = 0;
-	pid_t pid = optint(L, 1, -1);
+	pid_t pid = (pid_t)optinteger(L, 1, -1);
 	int options = optint(L, 2, 0);
 	checknargs(L, 2);
 
