@@ -1,6 +1,6 @@
 /*
  * POSIX library for Lua 5.1, 5.2, 5.3 & 5.4.
- * Copyright (C) 2013-2021 Gary V. Vaughan
+ * Copyright (C) 2013-2023 Gary V. Vaughan
  * Copyright (C) 2010-2013 Reuben Thomas <rrt@sc3d.org>
  * Copyright (C) 2008-2010 Natanael Copa <natanael.copa@gmail.com>
  * Clean up and bug fixes by Leo Razoumov <slonik.az@gmail.com> 2006-10-11
@@ -70,7 +70,7 @@ Terminate the calling process.
 static int
 P_exit(lua_State *L)
 {
-	pid_t ret = checkint(L, 1);
+	pid_t ret = (pid_t)checkinteger(L, 1);
 	checknargs(L, 1);
 	_exit(ret);
 	return 0; /* Avoid a compiler warning (or possibly cause one
@@ -128,7 +128,7 @@ Palarm(lua_State *L)
 {
 	int seconds = checkint(L, 1);
 	checknargs(L, 1);
-	return pushintresult(alarm(seconds));
+	return pushintegerresult(alarm(seconds));
 }
 /***
 Set the working directory.
@@ -448,12 +448,11 @@ static int
 Pgetcwd(lua_State *L)
 {
 #ifdef __GNU__
-	char *b = get_current_dir_name();
+	char *r = get_current_dir_name();
 	checknargs(L, 0);
-	if (b == NULL)
-		/* we return the same error as below */
-		return pusherror(L, ".");
-	return pushstringresult(b);
+	if (r != NULL)
+        lua_pushstring(L, r);
+    free(r);
 #else
 	long size = pathconf(".", _PC_PATH_MAX);
 	void *ud;
@@ -469,8 +468,8 @@ Pgetcwd(lua_State *L)
 	if (r != NULL)
 		lua_pushstring(L, b);
 	lalloc(ud, b, (size_t)size + 1, 0);
-	return (r == NULL) ? pusherror(L, ".") : 1;
 #endif
+	return (r == NULL) ? pusherror(L, ".") : 1;
 }
 
 
@@ -484,7 +483,7 @@ static int
 Pgetegid(lua_State *L)
 {
 	checknargs(L, 0);
-	return pushintresult(getegid ());
+	return pushintegerresult(getegid());
 }
 
 
@@ -498,7 +497,7 @@ static int
 Pgeteuid(lua_State *L)
 {
 	checknargs(L, 0);
-	return pushintresult(geteuid ());
+	return pushintegerresult(geteuid());
 }
 
 
@@ -512,7 +511,7 @@ static int
 Pgetgid(lua_State *L)
 {
 	checknargs(L, 0);
-	return pushintresult(getgid ());
+	return pushintegerresult(getgid());
 }
 
 
@@ -624,8 +623,8 @@ Pgetopt(lua_State *L)
 	checknargs(L, 4);
 	checktype(L, 1, LUA_TTABLE, "list");
 	optstring = luaL_checkstring(L, 2);
-	opterr = optint(L, 3, 0);
-	optind = optint(L, 4, 1);
+	opterr = optinteger(L, 3, 0);
+	optind = optinteger(L, 4, 1);
 
 	argc = (int)lua_objlen(L, 1) + 1;
 
@@ -658,7 +657,7 @@ static int
 Pgetpgrp(lua_State *L)
 {
 	checknargs(L, 0);
-	return pushintresult(getpgrp ());
+	return pushintegerresult(getpgrp());
 }
 
 
@@ -671,7 +670,7 @@ static int
 Pgetpid(lua_State *L)
 {
 	checknargs(L, 0);
-	return pushintresult(getpid ());
+	return pushintegerresult(getpid());
 }
 
 
@@ -685,7 +684,7 @@ static int
 Pgetppid(lua_State *L)
 {
 	checknargs(L, 0);
-	return pushintresult(getppid ());
+	return pushintegerresult(getppid());
 }
 
 
@@ -699,7 +698,7 @@ static int
 Pgetuid(lua_State *L)
 {
 	checknargs(L, 0);
-	return pushintresult(getuid ());
+	return pushintegerresult(getuid());
 }
 
 
@@ -716,7 +715,7 @@ Pgethostid(lua_State *L)
 {
 	checknargs(L, 0);
 #if HAVE_GETHOSTID
-	return pushintresult(gethostid());
+	return pushintegerresult(gethostid());
 #else
 	lua_pushnil(L);
 	lua_pushliteral(L, "unsupported by this host");
@@ -842,7 +841,7 @@ static int
 Plseek(lua_State *L)
 {
 	int fd = checkint(L, 1);
-	int offset = checkint(L, 2);
+	off_t offset = (off_t)checkinteger(L, 2);
 	int whence = checkint(L, 3);
 	checknargs(L, 3);
 	return pushresult(L, lseek(fd, offset, whence), NULL);
@@ -886,7 +885,7 @@ Ppathconf(lua_State *L)
 {
 	const char *path = luaL_checkstring(L, 1);
 	checknargs(L, 2);
-	return pushintresult(pathconf(path, checkint(L, 2)));
+	return pushintegerresult(pathconf(path, checkint(L, 2)));
 }
 
 
@@ -931,7 +930,7 @@ static int
 Pread(lua_State *L)
 {
 	int fd = checkint(L, 1);
-	int count = checkint(L, 2), ret;
+	size_t count = (size_t)checkinteger(L, 2), ret;
 	void *ud, *buf;
 	lua_Alloc lalloc;
 
@@ -964,15 +963,13 @@ Read value of a symbolic link.
 static int
 Preadlink(lua_State *L)
 {
-	char *b;
+	char b[PATH_MAX];
 	struct stat s;
 	const char *path = luaL_checkstring(L, 1);
 	void *ud;
 	lua_Alloc lalloc;
-	ssize_t n, bufsiz;
-	int err;
+	ssize_t n;
 	checknargs(L, 1);
-	lalloc = lua_getallocf(L, &ud);
 
 	errno = 0; /* ignore outstanding unreported errors */
 
@@ -980,7 +977,6 @@ Preadlink(lua_State *L)
 	if (lstat(path, &s) < 0)
 		return pusherror(L, path);
 
-	/* diagnose non-symlinks */
 	if (!S_ISLNK(s.st_mode))
 	{
 		lua_pushnil(L);
@@ -989,30 +985,12 @@ Preadlink(lua_State *L)
 		return 3;
 	}
 
-	/* allocate a buffer for linkname, with no trailing \0 */
-	bufsiz = s.st_size > 0 ? s.st_size : PATH_MAX;
-	if ((b = lalloc(ud, NULL, 0, bufsiz)) == NULL)
-		return pusherror(L, "lalloc");
-
-	n = readlink(path, b, bufsiz);
-	err = errno; /* save readlink error code, if any */
-	if (n > 0)
-		lua_pushlstring(L, b, n);
-	lalloc(ud, b, bufsiz, 0);
-
-	/* report new errors from this function */
+	n = readlink(path, b, PATH_MAX);
 	if (n < 0)
-	{
-		errno = err; /* restore readlink error code */
+		/* report new errors from this function */
 		return pusherror(L, "readlink");
-	}
-	else if (n < s.st_size)
-	{
-		lua_pushnil(L);
-		lua_pushfstring(L, "%s: readlink wrote only %d of %d bytes", path, n, s.st_size);
-		return 2;
-	}
 
+	lua_pushlstring(L, b, n);
 	return 1;
 }
 
@@ -1072,8 +1050,8 @@ Psetpid(lua_State *L)
 			return pushresult(L, setsid(), NULL);
 		case 'p':
 		{
-			pid_t pid  = checkint(L, 2);
-			pid_t pgid = checkint(L, 3);
+			pid_t pid  = (pid_t)checkinteger(L, 2);
+			pid_t pgid = (pid_t)checkinteger(L, 3);
 			return pushresult(L, setpgid(pid,pgid), NULL);
 		}
 		default:
@@ -1097,7 +1075,7 @@ Psleep(lua_State *L)
 {
 	unsigned int seconds = checkint(L, 1);
 	checknargs(L, 1);
-	return pushintresult(sleep(seconds));
+	return pushintegerresult(sleep(seconds));
 }
 
 
@@ -1129,7 +1107,7 @@ static int
 Psysconf(lua_State *L)
 {
 	checknargs(L, 1);
-	return pushintresult(sysconf(checkint(L, 1)));
+	return pushintegerresult(sysconf(checkint(L, 1)));
 }
 
 
@@ -1146,7 +1124,7 @@ Name of a terminal device.
 static int
 Pttyname(lua_State *L)
 {
-	int fd=optint(L, 1, 0);
+	int fd = optint(L, 1, 0);
 	char *name;
 	checknargs(L, 1);
 	name = ttyname(fd);
@@ -1194,7 +1172,7 @@ static int
 Ptcsetpgrp(lua_State *L)
 {
 	int fd = checkint(L, 1);
-	int pgid = checkint(L, 2);
+	pid_t pgid = (pid_t)checkinteger(L, 2);
 	return pushresult(L, tcsetpgrp(fd, pgid), NULL);
 }
 #endif
@@ -1221,9 +1199,17 @@ Punlink(lua_State *L)
 
 /***
 Write bytes to a file.
+If *nbytes* is `nil` or omitted, write all bytes from *offset*
+through to the end of *buf*.  If *offset* is `nil` or omitted,
+start writing bytes from the beginning of *buf*.
+Bounds checks are enforced, returning `posix.errno.EINVAL`
+before attempting to write any bytes, if the requested
+parameters would access bytes outside *buf*.
 @function write
 @int fd the file descriptor to act on
 @string buf containing bytes to write
+@int[opt=#buf] nbytes number of bytes to write
+@int[opt=0] offset skip the first offset bytes of buf
 @treturn[1] int number of bytes written, if successful
 @return[2] nil
 @treturn[2] string error message
@@ -1233,12 +1219,37 @@ Write bytes to a file.
 static int
 Pwrite(lua_State *L)
 {
-	int fd = checkint(L, 1);
+	const int fd = checkint(L, 1);
 	const char *buf = luaL_checkstring(L, 2);
-	checknargs(L, 2);
-	return pushresult(L, write(fd, buf, lua_objlen(L, 2)), NULL);
+	const int buflen = lua_objlen(L, 2);
+	ssize_t nbytes = (ssize_t)optinteger(L, 3, buflen);
+	const off_t offset = (off_t)optinteger(L, 4, 0);
+	off_t invalid_offset = offset;
+
+	checknargs(L, 4);
+
+	/* `write(fd, buf, nil, #buf - 3)` -> write the last 3 bytes in buf */
+	if (offset && lua_type(L, 3) == LUA_TNIL)
+		nbytes = buflen - offset;
+
+	/* calling write with nbytes `0` may cause unspecified behaviour */
+	if (nbytes == 0)
+		return pushintegerresult(0);
+
+	if (offset >= 0 && nbytes > 0 && offset + nbytes <= buflen)
+		return pushresult(L, write(fd, buf + offset, nbytes), NULL);
+
+	if (offset + nbytes < 0 || offset + nbytes > buflen)
+		invalid_offset += nbytes;
+
+	errno = EINVAL;
+	lua_pushnil(L);
+	lua_pushfstring(L, "write: invalid attempt to access offset %d in a buffer of length %d", invalid_offset, buflen);
+	lua_pushinteger(L, errno);
+	return 3;
 }
 
+
 /***
 Truncate a file to a specified length.
 @function ftruncate
@@ -1254,7 +1265,7 @@ static int
 Pftruncate(lua_State *L)
 {
 	int fd = checkint(L, 1);
-	off_t length = checkint(L, 2);
+	off_t length = (off_t)checkinteger(L, 2);
 	checknargs(L, 2);
 	return pushresult(L, ftruncate(fd, length), NULL);
 }
@@ -1274,7 +1285,7 @@ static int
 Ptruncate(lua_State *L)
 {
 	const char *path = luaL_checkstring(L, 1);
-	off_t length = checkint(L, 2);
+	off_t length = (off_t)checkinteger(L, 2);
 	checknargs(L, 2);
 	return pushresult(L, truncate(path, length), NULL);
 }
